<?php
/**
* 	functionality for mailhandling.
*/

/**
*	Class MailMsg
*
*	Description:
*		Create and send mail (text/HTML) with simple attachment handling
*
*	Variables:
*		$sSubject		=	Subject of message.
*		$aTo			=	Array with addresses.
*		$bHasPlainPart		=	Has a Plain/Text part.
*		$bHasHTMLPart		=	Has a HTML/Text part.
*		$sPlainPart		=	The Plain part.
*		$sHTMLPart		=	The HTML part.
*		$sTo			=	String with all the TO-addresses.
*		$sToBCC			=	String with all the BCC-addresses.
*		$sToCC			=	String with all the CC-addresses.
*		$sFrom			=	Email send from.
*		$iPriority		=	Priority (default 3 - normal).
*		$sHeaders		=	Generated headers.
*		$sBody			=	Generated Body.
*		$aAttachments		=	Array with attachments.
*		$sBoundary		=	Boundary 1.
*		$sBoundary2		=	Boundary 2.
*/
class MailMsg{

	var $sSubject		=	"";
	var $aTo			=	"";
	var $bHasPlainPart	=	false;
	var $bHasHTMLPart	=	false;
	var $sPlainPart		=	"";
	var $sHTMLPart		=	"";
	var $sTo			=	"";
	var $sToBCC			=	"";
	var $sToCC			=	"";
	var $sFrom			=	"";
	var $iPriority		=	3;

	var $sHeaders		=	"";
	var $sBody			=	"";
	var $aAttachments	=	"";
	var $sBoundary		=	"";
	var $sBoundary2		=	"";

	/**
	* @name 	MailMsg
	* @desc		Constructor
	* @param  	str Subject of message.
	* @param 	str The senders email.
	* @param 	str The senders name.
	*/
	function MailMsg( $sSubject, $sFromEmail, $sFromName="" ){

		$this->sSubject		=	$this->sEncodeQP($sSubject);
		if($sFromName==""){
			$this->sFrom	=	$sFromEmail;
		}else{
			$this->sFrom	=	$this->sEncodeQP($sFromName)." <".$sFromEmail.">";
		}

		$this->aTo			=	array();
		$this->aAttachments	=	array();
		$this->sBoundary	=	uniqid("op5licensingSystem");
		$this->sBoundary2	=	uniqid("op5licensingSystem2");
	}

	/**
	*	@name 	sEncodeQP($sString){
	*	@desc	encodes to ISO8859-1
	*	@param  str input string
	*	@return encoded string
	*/
	function sEncodeQP($sString){
		$sText = '=?UTF-8?q?';
		for( $i=0;$i<strlen($sString);$i++){
			$val=ord($sString[$i]);
			if($val<128){
				$sText.=$sString[$i];
			}else{
				$val=dechex($val);
				$sText.='='.$val;
			}
		}
		$sText.='?=';
		return $sText;
	}

	/**
	* @name 	vAddRecipient
	* @desc		Adds a recipient to the mail
	* @param 	str Recipients Email
	* @param 	str [optional] What way to send (to,cc,bcc).
	* @param 	str [optional] The recipients name.
	*/
	function vAddRecipient( $sEmail, $sType="to", $sName="" ){

		if($sName==""){
			$sRecipient=$sEmail;
		}else{
			$sRecipient = $this->sEncodeQP($sName)." <".$sEmail.">";
		}
		$this->aTo[]=array("type"=>$sType,"name"=>$sRecipient);

	}

	/**
	* @name 	bAddPlainPart
	* @desc		Adds the plainpart to the email message
	* @param 	str the plain content
	* @return 	true on success
	*/
	function bAddPlainPart( $sContent ){
		$this->bHasPlainPart=true;
		$this->sPlainPart=$sContent;
	}

	/**
	* @name 	bAddHTMLPart
	* @desc		Adds the HTMLpart to the email message
	* @param  	the plain content
	* @return 	true on success
	*/
	function bAddHTMLPart( $sContent ){
		$this->bHasHTMLPart=true;
		$this->sHTMLPart=$sContent;
	}

	/**
	* @name 	vGenerateMailMsg()
	* @desc		Generates all the mainmessage information (headers, body, etc)
	*/
	function vGenerateMailMsg(){

		// GENERATE RECIPIENTS
		reset($this->aTo);
		$this->sTo		="";
		$this->sToBCC	="";
		$this->sToCC	="";

		while( list($iKey,$aReci)=each($this->aTo) ){
			switch($aReci["type"]){
				case "to":
					$this->sTo .= $aReci["name"]." ,";
					break;
				case "cc":
					$this->sToCC .= $aReci["name"]." ,";
					break;
				case "bcc":
					$this->sToBCC .= $aReci["name"]." ,";
					break;
			}
		}
		$this->sTo 		= ereg_replace(",$","",$this->sTo);
		$this->sToBCC 	= ereg_replace(",$","",$this->sToBCC);
		$this->sToCC 	= ereg_replace(",$","",$this->sToCC);


		// GENERATE HEADERS AND BODY
		$this->sHeaders = "";
		$this->sHeaders .= "From: ".$this->sFrom."\nReply-To: ".$this->sFrom."\n";
//		$this->sHeaders .= "To: ".$this->sTo."\n";
		if($this->sToCC!=""){$this->sHeaders .= "cc: ".$this->sToCC."\n";}
		if($this->sToBCC!=""){$this->sHeaders .= "bcc: ".$this->sToBCC."\n";}
		$this->sHeaders .= "X-Mailer: MailMsg-class (op5licensingSystem)\n";
		$this->sHeaders .= "X-Priority: ".$this->iPriority."\n";
		$this->sHeaders .= "MIME-Version: 1.0\n";
		$this->sBody = "";

		if(count($this->aAttachments)==0){
			if($this->bHasPlainPart && $this->bHasHTMLPart){
				$this->sHeaders .= "Content-Type: multipart/alternative;\n\tboundary=\"".$this->sBoundary."\"\r\n";

				$this->sBody .= "--".$this->sBoundary."\nContent-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
				$this->sBody .= $this->sPlainPart."\r\n\r\n";
				$this->sBody .= "--".$this->sBoundary."\nContent-Type: text/html; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
				$this->sBody .= $this->sHTMLPart."\r\n\r\n";
			}else{
				if($this->bHasPlainPart){
					$this->sHeaders .= "Content-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\n";
					$this->sBody = $this->sPlainPart;
				}else{
					$this->sHeaders .= "Content-Type: text/html; charset=UTF-8\nContent-Transfer-Encoding: 8bit\n";
					$this->sBody = $this->sHTMLPart;
				}
			}
		}else{
			$this->sHeaders .= "Content-Type: multipart/mixed;\n\tboundary=\"".$this->sBoundary."\"\r\n";
			if($this->bHasPlainPart && $this->bHasHTMLPart){
				$this->sBody .= "--".$this->sBoundary."\n";
				$this->sBody .= "Content-Type: multipart/alternative; boundary=".$this->sBoundary2."\r\n\r\n";
				$this->sBody .= "--".$this->sBoundary2."\nContent-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
				$this->sBody .= $this->sPlainPart."\r\n\r\n";
				$this->sBody .= "--".$this->sBoundary2."\nContent-Type: text/html; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
				$this->sBody .= $this->sHTMLPart."\r\n\r\n";
				$this->sBody .= "--".$this->sBoundary2."--\r\n\r\n";

			}else{
				if($this->bHasPlainPart){
					$this->sBody .= "--".$this->sBoundary."\n";
					$this->sBody .= "Content-Type: text/plain; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
					$this->sBody .= $this->sPlainPart."\r\n\r\n";
				}else{
					$this->sBody .= "--".$this->sBoundary."\n";
					$this->sBody .= "Content-Type: text/html; charset=UTF-8\nContent-Transfer-Encoding: 8bit\r\n\r\n";
					$this->sBody .= $this->sHTMLPart."\r\n\r\n";
				}
			}
			// ADD THE ATTACHMENTS TO THE BODY
			reset($this->aAttachments);
			while( list($iKey,$aAtt) = each($this->aAttachments) ){

				$this->sBody .= "--".$this->sBoundary."\n";
				$this->sBody .= "Content-Type: application/octet-stream; name=\"".$aAtt["name"]."\"\nContent-Transfer-Encoding: base64\n";
				$this->sBody .= "Content-Disposition: attachment; filename=\"".$aAtt["name"]."\"\r\n\r\n";

				$oFile=fopen($aAtt["file"],"rb");
				$this->sBody .= chunk_split(base64_encode(fread($oFile,filesize($aAtt["file"]))));
				fclose( $oFile );
				$this->sBody .= "\r\n\r\n";
				$this->sBody .= "--".$this->sBoundary."--\r\n\r\n";
			}
		}
	}

	/**
	* @name 	bAddAttachment
	* @desc		Add an attachment to the message.
	* @param 	str File to send
	* @param 	[optional] Filename of the file you want to send (if diffrent then the real file).
	* @return 	true on success. False if file is'nt found
	*/
	function bAddAttachment( $sFile, $sFileName="" ){
		if(is_file($sFile)){
			if($sFileName==""){
				$sFileName=basename($sFile);
			}
			$this->aAttachments[]=array("file"=>$sFile,"name"=>$sFileName);
			return true;
		}else{
			return false;
		}
	}

	/**
	* @name 	bSendMail()
	* @desc		Sends the mailmessage.
	* @return 	true if the message is send successfully.
	*/
	function bSendMail(){

		$this->vGenerateMailMsg();
		//echo "<pre>".htmlspecialchars($this->sSubject."\n".$this->sHeaders."\n\n".$this->sBody)."</pre>";
		//echo $this->sTo."\n".$this->sSubject."\n".$this->sHeaders."\n\n";
		$hostname = exec('hostname --long');
		if (empty($hostname) && $hostname != '(none)') {
			// unable to get a valid hostname
			$return_path = 'monitor@localhost';
		} else {
			$return_path = 'monitor@'.$hostname;
		}
		return mail($this->sTo,$this->sSubject,$this->sBody,$this->sHeaders, '-f'.$return_path);

	}
}
?>
